@@ -29,7 +29,7 @@ namespace predictor {
 // List of files that will be force linked in static links.
 #ifdef XGBOOST_USE_CUDA
 DMLC_REGISTRY_LINK_TAG(gpu_predictor);
-#endif
+#endif  // XGBOOST_USE_CUDA
 DMLC_REGISTRY_LINK_TAG(cpu_predictor);
 }  // namespace predictor
 }  // namespace xgboost
